//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  DarkGray
#property indicator_color2  DarkGray
#property indicator_color3  DeepSkyBlue
#property indicator_color4  PaleVioletRed
#property indicator_width1  2
#property indicator_width3  2
#property indicator_width4  2
#property indicator_style2  STYLE_DOT
#property indicator_minimum 0
#property indicator_maximum 100

//
//
//
//
//

extern ENUM_TIMEFRAMES    TimeFrame        = PERIOD_CURRENT;
extern int                Length           = 32;
extern int                Smooth           =  3;
extern int                Signal           =  9;
extern ENUM_MA_METHOD     SignalMode       = MODE_EMA;
extern ENUM_STO_PRICE     Price            = STO_LOWHIGH;
extern double             HighLowStep      = 0.5;
extern bool               alertsOn         = false;
extern bool               alertsOnCurrent  = true;
extern bool               alertsMessage    = true;
extern bool               alertsSound      = false;
extern bool               alertsEmail      = false;
extern bool               alertsNotify     = false;
extern bool               ShowArrows       = false;
extern string             arrowsIdentifier = "corsto Arrows1";
extern double             arrowsUpperGap   = 0.5;
extern double             arrowsLowerGap   = 0.5;
extern color              arrowsUpColor    = LimeGreen;
extern color              arrowsDnColor    = Red;
extern int                arrowsUpCode     = 241;
extern int                arrowsDnCode     = 242;


double stoch[];
double signal[];
double max[];
double min[];
double trend[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
// 
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,stoch);
   SetIndexBuffer(1,signal);
   SetIndexBuffer(2,max);
   SetIndexBuffer(3,min);
   SetIndexBuffer(4,trend);
      TimeFrame = MathMax(TimeFrame,_Period);
      IndicatorShortName(timeFrameToString(TimeFrame)+" Corridor stochastic ("+Length+","+Signal+","+Smooth+")");
   return(0);
}
int deinit() 
{  
   deleteArrows(); 
return(0); 
}


//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit=MathMin(MathMax(Bars-counted_bars,3*TimeFrame/_Period),Bars-1);

   //
   //
   //
   //
   //

   for(int i=limit; i>=0; i--)
   {
      int y     = iBarShift(NULL,TimeFrame,Time[i]);
      stoch[i]  = iStochastic(NULL,TimeFrame,Length,Signal,Smooth,SignalMode,Price,MODE_MAIN  ,y);
      signal[i] = iStochastic(NULL,TimeFrame,Length,Signal,Smooth,SignalMode,Price,MODE_SIGNAL,y);
      max[i]    = MathMax(stoch[i],max[i+1]-HighLowStep);
      min[i]    = MathMin(stoch[i],min[i+1]+HighLowStep);
      
      //
      //
      //
      //
      //
      
      trend[i] = trend[i+1];
         if (stoch[i]==max[i]) trend[i] =  1;
         if (stoch[i]==min[i]) trend[i] = -1;
         
         //
         //
         //
         //
         //
     
         if (ShowArrows)
         {
            deleteArrow(Time[i]);
            if (trend[i] != trend[i+1])
            {
               if (trend[i] == 1)  drawArrow(i,arrowsUpColor,arrowsUpCode,false);
               if (trend[i] ==-1)  drawArrow(i,arrowsDnColor,arrowsDnCode, true);
            }
          }
   }         
   manageAlerts();
   return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,TimeFrame,whichBar)); 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," "+timeFrameToString(TimeFrame)+" corridor stochastic trend changed to "+doWhat);
          if (alertsMessage) Alert(message);
          if (alertsNotify)  SendNotification(message);
          if (alertsEmail)   SendMail(Symbol()+" corridor stochastic",message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}
